import { widgetKeys } from './widgets';
/**
 * 对函数进行批量绑定
 * @param {*} funcObj
 * @param {*} context
 */
export function bindFuncObj(funcObj = {}, context) {
  return Object.keys(funcObj).reduce((obj, fnName) => {
    obj[fnName] = funcObj[fnName].bind(context);
    return obj;
  }, {});
}

/**
 * 用于处理自定义组件props传参结构，对系统变量进行保留
 */
export function resolveComponentProps(props, isPlainProps) {
  const { staticResourceAttribute = [] } = props;
  staticResourceAttribute.map((property) => {
    if (props.data && props.data[property]) {
      props.data[property] = getStaticResourceAttribute(props.data[property]);
    }
  });

  const { data = {}, events = [], ...restProps } = props;
  const customProps = { ...data };

  if (!isPlainProps) {
    widgetKeys.forEach((prop) => delete customProps[prop]);
    return { ...props, data: customProps };
  }

  const builtinProps = [
    // react 保留字
    'ref',
    'key',
    'dangerouslySetInnerHTML',
    'className',
    'htmlFor',
    'style',
    'contentEditable',
    // lowcode 保留字
    ...widgetKeys,
    // 小程序保留字
    'id',
    'class',
    'hidden',
    'slot',
  ];
  // delete builtin props
  // 选区events处理
  const { _selectableBlockEvents, ...restData } = data;
  builtinProps.forEach((prop) => delete restData[prop]);

  const { emit: _selectableBlockEmit } = _selectableBlockEvents;
  if (Array.isArray(_selectableBlockEvents.events)) {
    _selectableBlockEvents.events = new Proxy(
      _selectableBlockEvents.events.reduce((events, item) => {
        const propName = item;
        events[propName] = (e) => _selectableBlockEmit(propName, e);
        return events;
      }, {}),
      {
        get(obj, prop) {
          return prop in obj ? obj[prop] : (e) => _selectableBlockEmit(prop, e);
        },
      },
    );
  }
  return {
    ...restData,
    ...restProps,
    _selectableBlockEvents,
    events: new Proxy(
      events.reduce((events, item) => {
        const propName = item;
        events[propName] = (e) => restProps.emit(propName, e);
        return events;
      }, {}),
      {
        get(obj, prop) {
          return prop in obj ? obj[prop] : (e) => restProps.emit(prop, e);
        },
      },
    ),
  };
}

function getStaticResourceAttribute(staticUrl) {
  if (/^\//.test(staticUrl)) {
    const { domain = '' } = window.app || {};
    const url = `https://${domain}${staticUrl}`;
    return url;
  }
  return staticUrl;
}
